"""Extractor for genius.com."""

import logging

from ..extractor import LyricsExtractor
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class Genius(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "Genius"
    url = "https://genius.com/"
    display_url = "genius.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        bs = url_data.bs

        lyrics_window = bs.find_all("div", {"class": "lyrics"})[0]
        lyrics = lyrics_window.text

        title = bs.find("h1", attrs={"class": "header_with_cover_art-primary_info-title"}).text.strip()

        return Lyrics(title, lyrics)



"""Lyrics object."""
import json
import time
from io import IOBase

from .. import utils


class LyricsOrigin:
    """Represents a place where lyrics come from."""

    __slots__ = ["query", "url", "source_name", "source_url"]

    def __init__(self, url, source_name, source_url, *, query=None):
        """Create new origin."""
        self.url = url
        self.source_name = source_name
        self.source_url = source_url
        self.query = query

    def __str__(self):
        """Return string rep."""
        return self.source_name

    @classmethod
    def from_dict(cls, data):
        """Load from dict."""
        return cls(**data)

    def to_dict(self):
        """Convert to dict."""
        return {
            "query": self.query,
            "url": self.url,
            "source_name": self.source_name,
            "source_url": self.source_url
        }


class Lyrics:
    """Represents lyrics for a song."""

    __slots__ = ["title", "lyrics", "origin", "timestamp"]

    def __init__(self, title, lyrics, *, timestamp=None):
        """Create lyrics."""
        self.title = title
        self.lyrics = lyrics
        self.origin = None

        self.timestamp = timestamp or time.time()

    def __str__(self):
        """Return string rep."""
        return "<Lyrics for \"{}\" from {}>".format(self.title, self.origin)

    @property
    def save_name(self):
        """Get a possible filename."""
        return utils.safe_filename(self.origin.query or self.title)

    @classmethod
    def from_dict(cls, data):
        """Load from dict."""
        data["origin"] = LyricsOrigin.from_dict(data["origin"])
        return cls(**data)

    def to_dict(self):
        """Convert to dict."""
        return {
            "title": self.title,
            "lyrics": self.lyrics,
            "origin": self.origin.to_dict(),
            "timestamp": self.timestamp
        }

    def save(self, f=None):
        """Save the lyrics."""
        if isinstance(f, IOBase):
            d = f
        elif isinstance(f, str):
            d = open(f, "w+")
        else:
            d = open(self.save_name, "w+")

        json.dump(self.to_dict(), d)
        d.seek(0)

        return d



"""Setup for this stuff."""

from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="lyricsfinder",
    version="1.0.1",
    description="Retrieve lyrics for a song",
    long_description=long_description,
    url="https://github.com/siku2/LyricsFinder",
    author="siku2",
    author_email="siku2@outlook.com",
    classifiers=[
        "Development Status :: 4 - Beta",

        "Intended Audience :: Developers",
        "Topic :: Multimedia :: Sound/Audio",

        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6"
    ],
    keywords="lyrics",
    packages=find_packages(exclude=["docs", "tests"]),
    install_requires=["beautifulsoup4", "requests"]
)



"""Extractor for animelyrics.com."""

import logging
import re

import requests

from ..extractor import LyricsExtractor
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class Animelyrics(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "Animelyrics"
    url = "http://www.animelyrics.com/"
    display_url = "animelyrics.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        bs = url_data.bs

        main_body = bs.find_all("table")[0]
        lyrics_window = main_body.find_all("table")

        if lyrics_window:  # shit's been translated
            log.info("these lyrics have been translated... sighs...")
            lyrics_window = lyrics_window[0]

            lines = lyrics_window.find_all("tr")
            lyrics = ""
            for line in lines:
                p = line.td
                if p:
                    p.span.dt.replace_with("")
                    for br in p.span.find_all("br"):
                        br.replace_with("\n")

                    lyrics += p.span.text
            lyrics = lyrics.strip()
        else:
            raw = requests.get(re.sub(r"\.html?", ".txt", url_data.url), allow_redirects=False)
            content = raw.text.strip()
            match = re.search(r"-{10,}(.+?)-{10,}", content, flags=re.DOTALL)
            if match:
                lyrics = match.group(1).strip()

        title = bs.find("td", attrs={"valign": "top"}).find("h1").text.strip()

        return Lyrics(title, lyrics)
